import java.awt.Container;
import java.awt.GridLayout;
import javax.swing.JApplet;
import javax.swing.JPanel;
import javax.swing.JLabel;
import java.util.Random;

public class AppletTest extends JApplet {

    private Set e1 ;
    private Set e2 ;
    private Set e3 ;
    private Random random;
        
    /**
     * Called by the browser or applet viewer to inform this Applet that it
     * has been loaded into the system. 
     */
    public void init() {
        random = new Random();
        // Test the class Set and store the results in the attributes of this class
        testSet();
        // Set the graphical content pane and let the browser display
        setContentPane(makeContentPane()) ;
    } // end init()

    /**
     * For testing the class Set :
     *      create two random sets e1 and e2
     *      create the intersection set e3
     */
    public void testSet() {
	  final int CAPAMAX1 =  8 ; // arbitrary limit of capacity for the set e1
        final int CAPAMAX2 = 12 ; // arbitrary limit of capacity for the set e2

        e1 = createSet(CAPAMAX1);
        e2 = createSet(CAPAMAX2);
        e3 = e1.inter(e2);
    } // end testSet()

    /**
     * Creates a random set
     * @param	maximum capacity
     * @return	a set of random elements, 
     *		the number of elements being random
     */
    private Set createSet(int maxCapacity ) {
        int capacity = 1 + random.nextInt(maxCapacity);
        Set e = new Set(capacity);
        int nbOfElements = random.nextInt(capacity+1);
        
	  // add at most nbOfElements elements (possibly zero) to the set e 
        for(int i=1; i<=nbOfElements; i++) 
		e.addExtended(random.nextInt(capacity+1));
        
        return e;

    } // end createSet()
    
    /**
     * Build the graphical pane
     */
    public Container makeContentPane() {
        int i ;
        int maxNbOfElements = Math.max(e1.cardinal(), e2.cardinal());
        
        // Create the graphical container and define the layout
        JPanel pane = new JPanel();
        pane.setLayout(new GridLayout(4,                    // number of rows
                                      maxNbOfElements+3));  // number of columns

        // Add widgets to container
        pane.add(new JLabel("Set")); 
        pane.add(new JLabel("Capacity")); 
        pane.add(new JLabel("Cardinal")); 
        for (i=1; i<=maxNbOfElements; i++) pane.add(new JLabel("x"+i));
        
        addToPane(pane, e1, "e1", maxNbOfElements);        
        addToPane(pane, e2, "e2", maxNbOfElements);        
        addToPane(pane, e3, "e3", maxNbOfElements);        

        return pane;

    } // end makeContentPane()

    /**
     * Add a given set description to a panel
     */
    private void addToPane(JPanel p, Set e, String s, int maxNbOfElements) {
        p.add(new JLabel(s)); 
        p.add(new JLabel(""+e.capacity())); 
        p.add(new JLabel(""+e.cardinal())); 
        for(int i=0; i<=e.capacity(); i++) 
            if (e.isElement(i)) p.add(new JLabel(""+i));
        for(int i=e.cardinal()+1; i<=maxNbOfElements; i++)  
            p.add(new JLabel(""));
    } // end displaySet()

} // end class AppletTest
